//////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2007-2012, Image Engine Design Inc. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//
//     * Neither the name of Image Engine Design nor the names of any
//       other contributors to this software may be used to endorse or
//       promote products derived from this software without specific prior
//       written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
//  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//////////////////////////////////////////////////////////////////////////

#ifndef IECOREGL_TEXTURE_H
#define IECOREGL_TEXTURE_H

#include "IECoreGL/Bindable.h"
#include "IECoreGL/Export.h"
#include "IECoreGL/GL.h"

#include "IECoreImage/ImagePrimitive.h"

namespace IECoreGL
{

IE_CORE_FORWARDDECLARE( FrameBuffer );

/// The Texture class represents a reference counted OpenGL texture.
/// When the Texture object dies, it also removes the associated GL texture.
/// Constructors from various IECore datatypes are provided.
/// \todo Provide some control over resizing (to power of 2), mipmapping,
/// filtering etc. Rather than complicate the constructors in the subclasses
/// we should probabaly make ToGLTextureConverters with a bunch of parameters.
/// \todo Review Y axis ordering for Cortex 8 - currently a texture lookup in
/// GLSL ends up with the image upside down from the expected, but perhaps there
/// are other places relying on the current ordering? See TestTexture.performShaderParameterTest()
/// for a test that should be checking this but isn't.
class IECOREGL_API Texture : public Bindable
{
	public :

		friend class FrameBuffer;

		IE_CORE_DECLARERUNTIMETYPEDEXTENSION( IECoreGL::Texture, TextureTypeId, Bindable );

		/// Creates a texture object to wrap the already created
		/// GL texture specified.
		Texture( GLuint texture );
		~Texture() override;

		/// Binds the texture as the current GL texture.
		void bind() const override;

		class IECOREGL_API ScopedBinding
		{

			public :

				ScopedBinding( const Texture &texture );
				~ScopedBinding();

			private :

				GLint m_prevTexture;

		};

		/// Creates an ImagePrimitive using the texture contents.
		virtual IECoreImage::ImagePrimitivePtr imagePrimitive() const;

	protected :

		Texture();

		/// Derived classes must set this in their constructor.
		GLuint m_texture;

};

IE_CORE_DECLAREPTR( Texture );

} // namespace IECoreGL

#endif // IECOREGL_TEXTURE_H
